from gql import Client
from datetime import datetime
from typing import BinaryIO
from gql.transport.aiohttp import AIOHTTPTransport
from gql.transport.websockets import WebsocketsTransport
from adc_sdk import queries, exceptions
from typing import Iterator


class ADCClient:
    """
    Client class to use in order to interact with Argonne Discovery Cloud API.
    """

    def __init__(self, token: str):
        """
        Create an ADCClient instance to interact with the API.
        Arguments:
            token: API Access Token
        """
        self.token = token
        self.http_url = "https://rdm-stage.discoverycloud.anl.gov/graphql/"
        self.ws_url = "wss://rdm-stage.discoverycloud.anl.gov/graphql/"
        self.headers = {"authorization": f"JWT {token}"}
        self.client = Client(
            transport=AIOHTTPTransport(url=self.http_url, headers=self.headers)
        )
        self.ws_client = Client(
            transport=WebsocketsTransport(url=self.ws_url, headers=self.headers)
        )

    def _execute(self, query_cls, variables=None, file_upload=False):
        raw_response = self.client.execute(query_cls.query, variable_values=variables, upload_files=file_upload)
        response = raw_response[query_cls.path] if query_cls.path else raw_response
        self._check_for_errors(response)
        return response

    def get_tokens(self) -> dict:
        """
        Retrieve the current user's tokens.
        """
        return self._execute(queries.TOKENS)

    def get_studies(self) -> dict:
        """
        Retrieve studies the current user has permission over.
        """
        return self._execute(queries.STUDIES)

    def get_study(self, study_id: str) -> dict:
        """
        Retrieve a specific study.
        Arguments:
            study_id: Study ID
        """
        variables = {"id": study_id}
        return self._execute(queries.STUDY, variables)

    def get_sample(self, sample_id: str) -> dict:
        """
        Retrieve a specific sample.
        Arguments:
            sample_id: Sample ID
        """
        variables = {"id": sample_id}
        return self._execute(queries.SAMPLE, variables)

    def get_datafile(self, datafile_id: str) -> dict:
        """
        Retrieve a specific datafile.
        Arguments:
            datafile_id: Datafile ID
        """
        variables = {"id": datafile_id}
        return self._execute(queries.DATAFILE, variables)

    def get_job(self, job_id: str) -> dict:
        """
        Retrieve a specific job.
        Arguments:
            job_id: Job ID
        """
        variables = {"id": job_id}
        return self._execute(queries.JOB, variables)

    def get_current_user(self) -> dict:
        """
        Retrieve the currently authenticated user.
        """
        return self._execute(queries.CURRENT_USER)

    def get_investigation(self, investigation_id: str) -> dict:
        """
        Retrieve a specific investigation.
        Arguments:
            investigation_id: Investigation ID
        """
        variables = {"id": investigation_id}
        return self._execute(queries.INVESTIGATION, variables)

    def create_token(self, name: str) -> dict:
        """
        Create a new access token for the currently logged user.
        Arguments:
            name: Desired token name
        """
        variables = {"name": name}
        return self._execute(queries.CREATE_TOKEN, variables)

    def delete_token(self, token_id: str) -> dict:
        """
        Delete a specific access token for the current user.
        Arguments:
            token_id: ID of the token that will be deleted
        """
        variables = {"tokenId": token_id}
        return self._execute(queries.DELETE_TOKEN, variables)

    def create_study(self, name: str, description: str, keywords: list = None) -> dict:
        """
        Create a new study.
        Arguments:
            name: study name
            description: study description
            keywords: study keywords
        """
        variables = {
            "description": description,
            "keywords": keywords if keywords else [],
            "name": name,
        }
        return self._execute(queries.CREATE_STUDY, variables)

    def create_sample(
        self, file: BinaryIO, study_id: str, name: str, keywords: list = None, parent_id: str = None, source: str = None
    ) -> dict:
        """
        Create a new sample.
        Arguments:
            file: sample file, opened as binary (i.e. with 'rb')
            study_id: study id
            name: sample name
            keywords: sample keywords
            parent_id: id of parent sample
            source: custom additional string value
        """
        variables = {
            "file": file,
            "studyId": study_id,
            "name": name,
            "keywords": keywords if keywords else [],
        }
        if parent_id: variables["parentId"] = parent_id
        if source: variables["source"] = source
        return self._execute(queries.CREATE_SAMPLE, variables, file_upload=True)

    def create_datafile(
        self, name: str, job_id: str, file: BinaryIO, description: str = None, source: str = None
    ) -> dict:
        """
        Create a new datafile.
        Arguments:
            file: actual file, opened as binary (i.e. with 'rb')
            job_id: job id
            name: sample name
            description: datafile description
            source: custom additional string value
        """
        variables = {
            "file": file,
            "jobId": job_id,
            "name": name,
        }
        if description: variables["description"] = description
        if source: variables["source"] = source
        return self._execute(queries.CREATE_DATAFILE, variables, file_upload=True)

    def create_investigation(
        self, study_id: str, name: str, description: str, keywords: list = None, investigation_type: str = None
    ) -> dict:
        """
        Create a new investigation.
        Arguments:
            study_id: study id
            name: investigation name
            description: investigation description
            keywords: investigation keywords
            investigation_type: available values are ['experiment', 'simulation', 'observation', 'measurement']
        """
        variables = {
            "studyId": study_id,
            "name": name,
            "description": description,
            "keywords": keywords if keywords else [],
        }
        if investigation_type: variables["investigationType"] = investigation_type
        return self._execute(queries.CREATE_INVESTIGATION, variables)

    def create_job(self, investigation_id: str, sample_id: str, start_datetime: datetime, end_datetime: datetime = None, status: str = None, source: str = None) -> dict:
        """
        Create a new job.
        Arguments:
            investigation_id: investigation id
            sample_id: id of sample to be used as job input
            start_datetime: job start datetime
            end_datetime: job end datetime
            status: available values are ['completed', 'cancelled', 'running']
            source: custom additional string value
        """
        variables = {
            "investigationId": investigation_id,
            "sampleId": sample_id,
            "startDatetime": start_datetime.isoformat(),
        }
        if end_datetime: variables["endDatetime"] = end_datetime.isoformat()
        if status: variables["status"] = status
        if source: variables["source"] = source
        return self._execute(queries.CREATE_JOB, variables)

    def update_job(self, job_id: str, status: str, end_datetime: datetime = None, source: str = None) -> dict:
        """
        Update an already existing job.
        Arguments:
            job_id: job id
            end_datetime: job end datetime
            status: available values are ['completed', 'cancelled', 'running']
            source: custom additional string value
        """
        variables = {
            "jobId": job_id,
            "status": status,
        }
        if end_datetime: variables["endDatetime"] = end_datetime.isoformat()
        if source: variables["source"] = source
        return self._execute(queries.UPDATE_JOB, variables)

    def set_permissions(self, study_id: str, user_id: str, permission_level: str) -> dict:
        """
        Set user permissions over a study.
        Arguments:
            study_id: study id
            user_id: id of the user we want to add / modify permissions over a study
            permission_level: available values are ['read', 'write', 'access']
        """
        variables = {
            "studyId": study_id,
            "userId": user_id,
            "permission": permission_level,
        }
        return self._execute(queries.SET_PERMISSIONS, variables)

    def remove_permissions(self, study_id: str, user_id: str) -> dict:
        """
        Remove user permissions over a study.
        Arguments:
            study_id: study id
            user_id: id of the user we want remove from a study
        """
        variables = {"studyId": study_id, "userId": user_id}
        return self._execute(queries.REMOVE_PERMISSIONS, variables)

    def subscribe_to_study(self, study_id: str) -> Iterator[dict]:
        """
        Subscribe to a study to get notifications when a new sample is added or an investigation is created.
        Arguments:
            study_id: study id
        """
        variables = {"studyId": study_id}
        for result in self.ws_client.subscribe(
            queries.STUDY_SUBSCRIPTION.query, variable_values=variables
        ):
            yield result

    def subscribe_to_investigation(self, investigation_id: str) -> Iterator[dict]:
        """
        Subscribe to an investigation to get notifications when a new job is created.
        Arguments:
            investigation_id: investigation id
        """
        variables = {"investigationId": investigation_id}
        for result in self.ws_client.subscribe(
                queries.INVESTIGATION_SUBSCRIPTION.query, variable_values=variables
        ):
            yield result

    def subscribe_to_job(self, job_id: str) -> Iterator[dict]:
        """
        Subscribe to a job to get notifications when a new datafile is created.
        Arguments:
            job_id: job id
        """
        variables = {"jobId": job_id}
        for result in self.ws_client.subscribe(
                queries.JOB_SUBSCRIPTION.query, variable_values=variables
        ):
            yield result

    @staticmethod
    def _check_for_errors(response):
        if "error" in response and response["error"]:
            raise exceptions.ADCError(response["error"])
        if "errors" in response:
            raise exceptions.ADCError(response["errors"][0]["message"])



import os
import json
import typer
from datetime import datetime
from adc_sdk.client import ADCClient
from adc_sdk.exceptions import ADCError
from typing import List

app = typer.Typer()


@app.callback()
def adc():
    """
    Argonne Discovery Cloud CLI
    """


def get_token_from_env():
    """
    Fetch and returns ADC_ACCESS_TOKEN env var.
    Outputs error if it is not present.
    """
    try:
        return os.environ["ADC_ACCESS_TOKEN"]
    except KeyError:
        message = "You need to set ADC_ACCESS_TOKEN environment variable before you can interact with ADC API"
        typer.echo(message, err=True)
        raise typer.Exit()


def get_client() -> ADCClient:
    """
    Return an instance of ADCClient using the ADC_ACCESS_TOKEN env var.
    """
    token = get_token_from_env()
    return ADCClient(token)

def print_to_stdout(content: dict):
    typer.echo(json.dumps(content, indent=4))


def fetch_and_output_response(client_method, *args):
    try:
        client = get_client()
        response = getattr(client, client_method)(*args)
        print_to_stdout(response)
    except ADCError as e:
        typer.echo(e.error, err=True)


@app.command()
def current_user():
    """
    Fetch current user's information.
    """
    client_method = 'get_current_user'
    fetch_and_output_response(client_method)


@app.command()
def tokens():
    """
    List current user's available API tokens.
    """
    client_method = 'get_tokens'
    fetch_and_output_response(client_method)


@app.command()
def studies():
    """
    List studies available to the current user.
    """
    client_method = 'get_studies'
    fetch_and_output_response(client_method)


@app.command()
def study(study_id: str):
    """
    Get details of a specific study.
    """
    client_method = 'get_study'
    fetch_and_output_response(client_method, study_id)


@app.command()
def investigation(investigation_id: str):
    """
    Get details of a specific investigation.
    """
    client_method = 'get_investigation'
    fetch_and_output_response(client_method, investigation_id)


@app.command()
def job(job_id: str):
    """
    Get details of a specific job.
    """
    client_method = 'get_job'
    fetch_and_output_response(client_method, job_id)


@app.command()
def datafile(datafile_id: str):
    """
    Get details of a specific datafile.
    """
    client_method = 'get_datafile'
    fetch_and_output_response(client_method, datafile_id)


@app.command()
def sample(sample_id: str):
    """
    Get details of a specific sample.
    """
    client_method = 'get_sample'
    fetch_and_output_response(client_method, sample_id)


@app.command()
def create_token(name: str):
    """
    Create a new token for the current user.
    """
    client_method = 'create_token'
    fetch_and_output_response(client_method, name)


@app.command()
def delete_token(token_id: str):
    """
    Delete a specific token from the current user's available tokens.
    """
    client_method = 'delete_token'
    fetch_and_output_response(client_method, token_id)


@app.command()
def create_study(
    name: str,
    description: str,
    keywords: List[str] = typer.Option(default=None),
):
    """
    Create a new Study, current user will be the owner and will have admin permissions.
    """
    keywords = [] if not keywords else [k for k in keywords[0].split(' ')]
    client_method = 'create_study'
    fetch_and_output_response(client_method, name, description, keywords)


@app.command()
def create_sample(
    sample_file_path: str,
    study_id: str,
    name: str,
    keywords: List[str] = typer.Option(default=None),
    parent_id: str = typer.Option(default=None),
    source: str = typer.Option(default=None),
):
    """
    Create a Sample under the specified Study to (possibly) be used as input for a Job to generate Datafiles.
    Triggers notifications for Study subscriptors.
    """
    if not os.path.isfile(sample_file_path):
        typer.echo("Invalid sample file path", err=True)
    keywords = [] if not keywords else [k for k in keywords[0].split(' ')]
    with open(sample_file_path, 'rb') as file:
        client_method = 'create_sample'
        fetch_and_output_response(client_method, file, study_id, name, keywords, parent_id, source)


@app.command()
def create_datafile(
    file_path: str,
    job_id: str,
    name: str,
    description: str = typer.Option(default=None),
    source: str = typer.Option(default=None),
):
    """
    Create a Datafile that will be considered as a result file from the specified Job
    Triggers notifications for Job subscriptors.
    """
    if not os.path.isfile(file_path):
        typer.echo("Invalid sample file path", err=True)
    with open(file_path, 'rb') as file:
        client_method = 'create_datafile'
        fetch_and_output_response(client_method, name, job_id, file, description, source)


@app.command()
def create_investigation(
    study_id: str,
    name: str,
    description: str,
    keywords: List[str] = typer.Option(default=None),
    investigation_type: str = typer.Option(default=None),
):
    """
    Create an Investigation under the specified Study.
    Triggers notifications for Study subscriptors.
    """
    client_method = 'create_investigation'
    keywords = [] if not keywords else [k for k in keywords[0].split(' ')]
    fetch_and_output_response(client_method, study_id, name, description, keywords, investigation_type)


@app.command()
def create_job(
    investigation_id: str,
    sample_id: str,
    start_datetime: datetime,
    end_datetime: datetime = typer.Option(default=None),
    status: str = typer.Option(default=None),
    source: str = typer.Option(default=None)
):
    """
    Create a Job under the specified Investigation.
    Triggers notifications for Investigation subscriptors.
    """
    client_method = 'create_job'
    fetch_and_output_response(
        client_method,
        investigation_id,
        sample_id,
        start_datetime,
        end_datetime,
        status,
        source
    )


@app.command()
def update_job(
    job_id: str,
    status: str,
    end_datetime: datetime = typer.Option(default=None),
    source: str = typer.Option(default=None)
):
    """
    Update an already existing Job.
    Triggers notifications for Job subscriptors.
    """
    client_method = 'update_job'
    fetch_and_output_response(
        client_method,
        job_id,
        status,
        end_datetime,
        source
    )


@app.command()
def set_permissions(study_id: str, user_id: str, permission_level: str):
    """
    Set permission level for the specified User over the specified Study.
    """
    client_method = 'set_permissions'
    fetch_and_output_response(client_method, study_id, user_id, permission_level)


@app.command()
def remove_permissions(study_id: str, user_id: str):
    """
    Remove the specified User permissions over the specified Study.
    """
    client_method = 'remove_permissions'
    fetch_and_output_response(client_method, study_id, user_id)


@app.command()
def subscribe_to_study(study_id):
    """
    Subscribe to the specified study and print notifications to stdout.
    Notifications will be either for new Samples and Investigations under the specified Study.
    """
    client = get_client()
    for notification in client.subscribe_to_study(study_id):
        print_to_stdout(notification)


@app.command()
def subscribe_to_investigation(investigation_id):
    """
    Subscribe to the specified Investigation and print notifications to stdout.
    Notifications will be for Jobs created under the specified Study.
    """
    client = get_client()
    for notification in client.subscribe_to_investigation(investigation_id):
        print_to_stdout(notification)


@app.command()
def subscribe_to_job(job_id):
    """
    Subscribe to the specified Job and print notifications to stdout.
    Notifications will be for Job updates and new Datafiles added to the specified Job.
    """
    client = get_client()
    for notification in client.subscribe_to_job(job_id):
        print_to_stdout(notification)



from gql import gql
from adc_sdk import raw_queries


class ADCQuery:

    def __init__(self, query, path=None):
        self.raw_query = query
        self.query = gql(self.raw_query)
        self.path = path


# Queries
CURRENT_USER = ADCQuery(raw_queries.CURRENT_USER)
TOKENS = ADCQuery(raw_queries.TOKENS)
STUDIES = ADCQuery(raw_queries.STUDIES)
STUDY = ADCQuery(raw_queries.STUDY)
SAMPLE = ADCQuery(raw_queries.SAMPLE)
DATAFILE = ADCQuery(raw_queries.DATAFILE)
JOB = ADCQuery(raw_queries.JOB)
INVESTIGATION = ADCQuery(raw_queries.INVESTIGATION)

# Mutations
CREATE_TOKEN = ADCQuery(raw_queries.CREATE_TOKEN, 'createToken')
DELETE_TOKEN = ADCQuery(raw_queries.DELETE_TOKEN, 'deleteToken')
CREATE_STUDY = ADCQuery(raw_queries.CREATE_STUDY, 'createStudy')
CREATE_SAMPLE = ADCQuery(raw_queries.CREATE_SAMPLE, 'createSample')
CREATE_JOB = ADCQuery(raw_queries.CREATE_JOB, 'createJob')
CREATE_DATAFILE = ADCQuery(raw_queries.CREATE_DATAFILE, 'createDatafile')
UPDATE_JOB = ADCQuery(raw_queries.UPDATE_JOB, 'updateJob')
SET_PERMISSIONS = ADCQuery(raw_queries.SET_PERMISSIONS, 'setPermissions')
REMOVE_PERMISSIONS = ADCQuery(raw_queries.REMOVE_PERMISSIONS, 'deletePermissions')
CREATE_INVESTIGATION = ADCQuery(raw_queries.CREATE_INVESTIGATION, 'createInvestigation')

# Subscription
STUDY_SUBSCRIPTION = ADCQuery(raw_queries.STUDY_SUBSCRIPTION)
INVESTIGATION_SUBSCRIPTION = ADCQuery(raw_queries.INVESTIGATION_SUBSCRIPTION)
JOB_SUBSCRIPTION = ADCQuery(raw_queries.JOB_SUBSCRIPTION)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['adc_sdk']

package_data = \
{'': ['*']}

install_requires = \
['aiohttp>=3.7.4,<4.0.0',
 'gql==3.0.0a6',
 'requests>=2.26.0,<3.0.0',
 'typer>=0.3.2,<0.4.0',
 'websockets>=9.1,<10.0']

entry_points = \
{'console_scripts': ['adc = adc_sdk.cli:app']}

setup_kwargs = {
    'name': 'adc-sdk',
    'version': '0.1.1',
    'description': 'Argonne Discovery Cloud SDK and CLI',
    'long_description': "# Argonne Discovery Cloud SDK & CLI\n\n## Docs: \nhttps://stage.discoverycloud.anl.gov/docs/sdk/\n\n## Installation\n\n* When positioned in this repo's root directory:\n```\npip install adc-sdk\n```\n\n## CLI Commands\n```\nadc create-datafile\nadc create-investigation\nadc create-job\nadc create-sample\nadc create-study\nadc create-token\nadc current-user\nadc datafile\nadc delete-token\nadc investigation\nadc job\nadc remove-permissions\nadc sample\nadc set-permissions\nadc studies\nadc study\nadc subscribe-to-investigation\nadc subscribe-to-job\nadc subscribe-to-study\nadc tokens\nadc update-job\n```\nYou can run `adc <command> --help` for more information.\n\n\n## License\nThis project is licensed under the Apache License - see the [LICENSE](LICENSE) file for details\n",
    'author': 'Argonne National Laboratory',
    'author_email': 'discovery@anl.gov',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



CURRENT_USER = """
    query {
        me {
            email
            name
            globusUsername
            organization
            created
            updated
            id
        }
    }
"""

STUDIES = """
    query {
        studies {
            edges {
                node {
                    id
                    name
                    description
                    keywords
                    startDate
                    status
                    created
                    updated
                    permissions { edges { node { user { id name email } level } } }
                    investigations { edges { node { id name description type keywords startDatetime endDatetime created updated } } }
                    samples { edges { node { id name url keywords created updated } } }
                }
            }
        }
    }
"""

STUDY = """
    query ($id: ID!) {
        study(id: $id) {
            id
            name
            description
            keywords
            startDate
            status
            created
            updated
            permissions { edges { node { user { id name email } level } } }
            investigations { edges { node { id name description type keywords startDatetime endDatetime created updated } } }
            samples { edges { node { id name url keywords created updated } } }
        }
    }
"""

CREATE_STUDY = """
    mutation ($description: String, $keywords: [String], $name: String) {
        createStudy(description: $description, keywords: $keywords, name: $name) {
            success
            error
            study {
                id
                name
                description
                keywords
                startDate
                status
                created
                updated
                permissions { edges { node { user { id name email } level } } }
                investigations { edges { node { id name description type keywords startDatetime endDatetime created updated } } }
                samples { edges { node { id name url keywords created updated } } }
            }
        }
    }
"""

CREATE_SAMPLE = """
    mutation ($file: Upload!, $keywords: [String], $name: String!, $parentId: ID, $source: String, $studyId: ID!) {
        createSample(file: $file, keywords: $keywords, name: $name, parentId: $parentId, source: $source, studyId: $studyId) {
            success
            error
            sample {
                id
                name
                user { id name email }
                keywords
                parent { id }
                created
                updated
                url
            }
        }
    }
"""

STUDY_SUBSCRIPTION = """
    subscription ($studyId: ID!) {
        study(studyId: $studyId) {
            study {
                id
                name
                description
                keywords
                startDate
                status
                permissions { edges { node { user { id name email } level } } }
                investigations { edges { node { id name description user { id name email }type keywords startDatetime endDatetime created updated } } }
                samples { edges { node { id name url keywords user { id name email } created } } }
                created
                updated
            }
            investigation { id name description type keywords startDatetime endDatetime created updated }
            sample { id name url user { id name email } keywords created updated }
            source
        }
    }
"""


TOKENS = """
    query {
        tokens{
            id
            name
        }
    }
"""

INVESTIGATION = """
    query ($id: ID!) {
        investigation(id: $id) {
            id
            name
            description
            type
            user { id name email }
            keywords
            study {
                id
                name
                description
                keywords
                startDate
                status
                created
                updated
                permissions { edges { node { user { id name email } level } } }
                investigations { edges { node { id name description type keywords startDatetime endDatetime created updated } } }
                samples { edges { node { id name url keywords created updated } } }
            }
            jobs { edges { node { id startDatetime endDatetime status  } } }
            startDatetime
            endDatetime
            created
            updated
        }
    }
"""

CREATE_INVESTIGATION = """
    mutation ($description: String!, $investigationType: String, $keywords: [String], $name: String!, $studyId: ID!) {
        createInvestigation(description: $description, investigationType: $investigationType, keywords: $keywords, name: $name, studyId: $studyId) {
            error
            success
            investigation {
                id
                name
                description
                type
                user { id name email }
                keywords
                study {
                    id
                    name
                    description
                    keywords
                    startDate
                    status
                    created
                    updated
                    permissions { edges { node { user { id name email } level } } }
                    investigations { edges { node { id name description type keywords startDatetime endDatetime created updated } } }
                    samples { edges { node { id name url keywords created updated } } }
                }
                startDatetime
                endDatetime
                created
                updated
            }
        }
    }
"""

DELETE_TOKEN = """
    mutation ($tokenId: ID) {
        deleteToken(tokenId: $tokenId) {
            success
            error
        }
    }
"""

SET_PERMISSIONS = """
    mutation ($permission: String, $studyId: ID, $userId: String) {
        setPermissions(permission: $permission, studyId: $studyId, userId: $userId) {
            success
            error
        }
    }
"""

REMOVE_PERMISSIONS = """
    mutation ($studyId: ID, $userId: ID) {
        removePermissions(studyId: $studyId, userId: $userId) {
            success
            error
        }
    }
"""

CREATE_TOKEN = """
    mutation ($name: String!) {
        createToken(name: $name){
            success
            error
            token
        }
    }
"""

CREATE_JOB = """
    mutation ($investigationId: ID!, $sampleId: ID!, $startDatetime: DateTime!, $endDatetime: DateTime, $status: String, $source: String) {
        createJob(investigationId: $investigationId, sampleId: $sampleId, startDatetime: $startDatetime, endDatetime: $endDatetime, status: $status, source: $source) {
            success
            error
            job {
                id
                endDatetime
                startDatetime
                status
                investigation { id name description type keywords startDatetime endDatetime created updated user { id name email } }
                sample { id name url user { id name email } keywords created updated }
                created
                updated
            }
        }
    }
"""

CREATE_DATAFILE = """
    mutation($name: String!, $jobId: ID!, $description: String, $file: Upload!, $source: String) {
        createDatafile(name: $name, jobId: $jobId, description: $description, file: $file, source: $source) {
            success
            error
            datafile {
              id
              name
              description
              url
              user { id name email }
            }
        }
    }
"""

UPDATE_JOB = """
    mutation ($jobId: ID!, $status: String!, $endDatetime: DateTime, $source: String){
        updateJob(jobId: $jobId, status: $status, endDatetime: $endDatetime, source: $source) {
            success
            error
            job {
                id
                startDatetime,
                endDatetime,
                status
                datafiles { edges { node { id name description url user { id name email } } } }
            }
        }
    }
"""

JOB_SUBSCRIPTION = """
    subscription ($jobId: ID!) {
        job(jobId: $jobId) {
            job {
                id
                endDatetime
                startDatetime
                status
                investigation { id name description type keywords startDatetime endDatetime created updated user { id name email } }
                sample { id name url user { id name email } keywords created updated }
                datafiles { edges { node { id name url user { id name email } created } } }
                created
                updated
            }
            datafile { id name url user { id name email } created }
            source
        }
    }
"""

INVESTIGATION_SUBSCRIPTION = """
    subscription ($investigationId: ID!) {
        investigation(investigationId: $investigationId) {
            job {
                id
                endDatetime
                startDatetime
                status
                sample { id name url user { id name email } keywords created }
                created
                updated
            }
            source
        }
    }
"""

SAMPLE = """
    query ($id: ID!){
        sample(id: $id){
            id
            name
            keywords
            url
            created
            user { id name email }
        }
    }
"""

DATAFILE = """
    query ($id: ID!) {
        datafile (id: $id) {
            id
            name
            description
            url
            user { id name email }
        }
    }
"""

JOB = """
    query ($id: ID!){
        job (id: $id){ 
            id
            startDatetime,
            endDatetime,
            status
            datafiles { edges { node { id name url user { id name email } created } } }
        }
    }
"""